# -*- coding: utf-8 -*-

from django.db.models import Q

from kinder.core.emie_unit.models import UnitDocFiles
from kinder.core.dict.models import DouType
from kinder.core.group.models import Pupil
from kinder.core.group.models import Group
from kinder.core.group.models import GroupStatusEnum
from kinder.core.unit.models import FilialData
from kinder.core.unit.models import UnitStatus
from kinder.core.unit.models import FilialTypeStatus
from kinder.core.unit.models import StructureTypes


ORGANIZATION_TYPES = {
    DouType.MUNICIPAL: 1,
    DouType.GOVERNMENT: 2,
    DouType.NOT_GOVERNMENT_WITH_LICENSE: 3,
    DouType.NOT_GOVERNMENT_WITHOUT_LICENSE: 4,
    DouType.IP_WITH_LICENSE: 5,
    DouType.IP_WITHOUT_LICENSE: 6,
    DouType.DEPARTMENTAL: 7
}


def get_dou_type(dou):

    dou_type = dou.dou_type
    if dou_type is None or dou_type.code == DouType.OTHER:
        code = DouType.GOVERNMENT
    else:
        code = dou_type.code
    return ORGANIZATION_TYPES[code]


WORK = 1
RECONSTRUCTION = 2
IN_CAPITAL_REPAIR = 3
SUSPENDED = 4
NO_CONTINGENT = 5
PENDING_OPEN = 6

STATUSES = {
    UnitStatus.RECONSTRUCTION: RECONSTRUCTION,
    UnitStatus.IN_CAPITAL_REPAIR: IN_CAPITAL_REPAIR,
    UnitStatus.SUSPENDED: SUSPENDED,
    UnitStatus.PENDING_OPEN: PENDING_OPEN
}


def get_dou_status(dou):

    filials = FilialData.objects.filter(
        head=dou, status=FilialTypeStatus.APPROVE,
        filial__closed=False, filial__gisdo__not_on_federal_report=False
    ).values_list("filial_id", flat=True).iterator()

    if Pupil.objects.filter(
        grup__status=GroupStatusEnum.FACT,
        temporary_deduct=False
    ).filter(
        Q(grup__unit=dou) |
        Q(grup__unit__in=filials)
    ).exists():
        status = WORK
    elif dou.status == UnitStatus.RECONSTRUCTION:
        status = STATUSES[UnitStatus.RECONSTRUCTION]
    elif dou.status == UnitStatus.IN_CAPITAL_REPAIR:
        status = STATUSES[UnitStatus.IN_CAPITAL_REPAIR]
    elif dou.status == UnitStatus.SUSPENDED:
        status = STATUSES[UnitStatus.SUSPENDED]
    elif dou.status == UnitStatus.PENDING_OPEN:
        status = STATUSES[UnitStatus.PENDING_OPEN]
    else:
        status = NO_CONTINGENT

    return status


def get_dou_structure_type(dou):

    conformity = {
        StructureTypes.KINDER: 1,
        StructureTypes.SCHOOL: 2,
        StructureTypes.PRESOTHER: 3,
        StructureTypes.UNIVERSITY: 4,
    }

    if dou.structure_type == StructureTypes.OTHER:
        return 1
    else:
        return conformity[dou.structure_type]


def get_groups_num(dou):

    filials = FilialData.objects.filter(
        head=dou, status=FilialTypeStatus.APPROVE,
        filial__closed=False, filial__gisdo__not_on_federal_report=False
    ).values_list("filial_id", flat=True).iterator()

    dou_groups = Group.extended_objects.with_count_norm().select_related(
        'type', 'health_need'
    ).filter(
        status=GroupStatusEnum.FACT
    ).filter(
        Q(unit=dou) | Q(unit__in=filials)
    )

    return dou_groups.count()


def get_filial_num(dou):
    u"""Количество подтвержденных филиалов."""

    return FilialData.objects.filter(
        head=dou, status=FilialTypeStatus.APPROVE
    ).count()


def get_unit_fias_house_guid(dou):
    u"""Возвращаем фиас код адреса учреждения."""

    if dou.address_house_guid:
        return dou.address_house_guid
    elif dou.address_street:
        return dou.address_street
    elif dou.address_place:
        return dou.address_place
    else:
        return "00000000-0000-0000-0000-000000000000"


def get_additional_info_by_dou(dou, model=UnitDocFiles):
    u"""Дополнительная информация по учреждению."""

    filials = FilialData.objects.filter(
        head=dou, status=FilialTypeStatus.APPROVE,
        filial__closed=False, filial__gisdo__not_on_federal_report=False
    ).values_list("filial_id", flat=True).iterator()
    extensions = model.objects.filter(
        Q(unit=dou) | Q(unit__in=filials)
    ).values_list('name', flat=True)

    if not extensions.exists():
        return u'нет'
    else:
        return ','.join(extensions)


def get_filial_info(dou):
    if FilialData.objects.filter(
            filial=dou, status=FilialTypeStatus.APPROVE).exists():
        return 1

    return 0
